import '../extensions.dart';
import 'executor.dart';
import 'isolate_executor.dart';

class PoolExecutor extends Executor {
  int _index = 0;
  late final List<IsolateExecutor> _delegates;

  PoolExecutor({required int concurrency}) {
    assert(concurrency > 0);
    _delegates = List.generate(concurrency, (index) => IsolateExecutor());
  }

  @override
  void dispose() {
    for (final delegate in _delegates) {
      delegate.dispose();
    }
  }

  @override
  bool get disposed => _delegates[0].disposed;

  @override
  List<Future<R>> submitAll<Q, R>(Job<Q, R> job) =>
      _delegates.map((delegate) => delegate.submit(job)).toList();

  @override
  Future<R> submit<Q, R>(Job<Q, R> job) => _nextDelegate(job).submit(job);

  Executor _nextDelegate(Job job) {
    final affinityDelegate = _delegates
        .where((delegate) => delegate.hasJobWithDeduplicationKey(job))
        .firstOrNull;
    if (affinityDelegate != null) {
      return affinityDelegate;
    }
    for (int attempt = 0; attempt < _delegates.length; ++attempt) {
      final delegate = _delegates[_nextIndex()];
      if (delegate.outstanding == 0) {
        return delegate;
      }
    }
    return _delegates[_nextIndex()];
  }

  int _nextIndex() {
    ++_index;
    if (_index == _delegates.length) {
      _index = 0;
    }
    return _index;
  }
}
